package org.opencalaccess;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Scanner;
import java.util.TimeZone;
import java.util.UUID;

import com.webobjects.foundation.NSArray;
import com.webobjects.foundation.NSDictionary;
import com.webobjects.foundation.NSMutableArray;

import er.extensions.foundation.ERXDictionaryUtilities;
import er.extensions.foundation.ERXValueUtilities;

public class U {

	public static final String LAPTOP = "rrk.";

	@SuppressWarnings("unchecked")
	public static NSDictionary<String, String> shortMonthToNum = ERXDictionaryUtilities
			.dictionaryWithObjectsAndKeys(new String[] {
					"01", "JAN",
					"02", "FEB",
					"03", "MAR",
					"04", "APR",
					"05", "MAY",
					"06", "JUN",
					"07", "JUL",
					"08", "AUG",
					"09", "SEP",
					"10", "OCT",
					"11", "NOV",
					"12", "DEC" });

	@SuppressWarnings("unchecked")
	public static NSDictionary<String, String> longMonthToNum = ERXDictionaryUtilities
			.dictionaryWithObjectsAndKeys(new String[] {
					"01", "January",
					"02", "February",
					"03", "March",
					"04", "April",
					"05", "May",
					"06", "June",
					"07", "July",
					"08", "August",
					"09", "September",
					"10", "October",
					"11", "November",
					"12", "December" });

	public static long now() {
		return System.currentTimeMillis();
	}

	public static boolean isNull(final Object obj) {
		return ERXValueUtilities.isNull(obj);
	}

	public static String nextUUID() {
		return UUID.randomUUID().toString();
	}

	public static boolean isLocal() {
		java.net.InetAddress localMachine;
		try {
			localMachine = java.net.InetAddress.getLocalHost();
		} catch (java.net.UnknownHostException e) {
			e.printStackTrace();
			throw new IllegalArgumentException("Cannot determine local machine name?");
		}
		return localMachine != null &&
				localMachine.getHostName() != null &&
				localMachine.getHostName().startsWith(U.LAPTOP);
	}

	public static boolean isRemote() {
		return !isLocal();
	}

	public static String dollarsForCents(Long cents) {
		Long small = cents % 100;
		Long large = cents / 100;
		StringBuilder str = new StringBuilder();
		str.append(large);
		str.append(".");
		if (small < 10) {
			str.append("0");
		}
		str.append(small);
		return str.toString();
	}

    public static final ThreadLocal<SimpleDateFormat> shortDateFormat = new ThreadLocal<SimpleDateFormat>() {
        @Override
        protected SimpleDateFormat initialValue() {
            SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd");
            fmt.setTimeZone(TimeZone.getTimeZone("UTC"));
            return fmt;
        }
    };

    public static final ThreadLocal<SimpleDateFormat> longDateFormatPST = new ThreadLocal<SimpleDateFormat>() {
        @Override
        protected SimpleDateFormat initialValue() {
            SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            fmt.setTimeZone(TimeZone.getTimeZone("PST"));
            return fmt;
        }
    };

    public static final ThreadLocal<SimpleDateFormat> longPathDateFormat = new ThreadLocal<SimpleDateFormat>() {
        @Override
        protected SimpleDateFormat initialValue() {
            SimpleDateFormat fmt = new SimpleDateFormat("yyyyMMdd_HHmm");
            fmt.setTimeZone(TimeZone.getTimeZone("UTC"));
            return fmt;
        }
    };

    private static String logStr(final Object... words) {
        String className = (new Throwable()).getStackTrace()[2].getClassName();
        StringBuilder s = new StringBuilder();
        s.append(longDateFormatPST.get().format(new Date()));
        s.append(" ");
        s.append(className);
        s.append(" ");
        for (Object str : words) {
            s.append(str);
        }
        return s.toString();
    }

    public static NSArray<String> linesInFile(final File file) {

        if (file == null) {
            System.err.println("FileNotFoundException: file is NULL");
        }

        NSMutableArray<String> lines = new NSMutableArray<>();

        Scanner s = null;
        try {
            s = new Scanner(file);
            while (s.hasNextLine()) {
                lines.add(s.nextLine());
            }
            s.close();
            return lines.immutableClone();
        } catch (java.io.FileNotFoundException e) {
            System.err.println("FileNotFoundException: for file: " + file.getAbsolutePath());
            return null;
        }
    }

    public static String stringInFile(final File file) {
    	return linesInFile(file).componentsJoinedByString("\n");
    }

    public static synchronized void log(final Object... words) {
        String s = logStr(words);
        System.out.println(s);
    }

    public static void err(final Object... words) {
        String s = logStr(words);
        System.err.println(s);
    }

    public static String string(final Object... strs) {
        StringBuilder s = new StringBuilder();
        for (Object str : strs) {
            s.append(str);
        }
        return s.toString();
    }
}
