package org.opencalaccess.boundaries.eo;


import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.webobjects.eoaccess.EOUtilities;
import com.webobjects.eocontrol.EOEditingContext;
import com.webobjects.foundation.NSArray;
import com.webobjects.foundation.NSDictionary;

import er.extensions.eof.ERXKey;
import er.extensions.eof.ERXKey.Type;

public class OsmRelation extends _OsmRelation {

	private static final long serialVersionUID = 5644160825448587726L;

	public static final ERXKey<org.opencalaccess.boundaries.eo.OsmRelationCheck> LAST_CHECK = new ERXKey<org.opencalaccess.boundaries.eo.OsmRelationCheck>("lastCheck", Type.Attribute);
	public static final ERXKey<String> LAST_UPDATE_DATE = new ERXKey<String>("lastUpdateDate", Type.Attribute);

	public static final String LAST_CHECK_KEY = LAST_CHECK.key();
	public static final String LAST_UPDATE_DATE_KEY = LAST_UPDATE_DATE.key();

	public static final String RELATION_API_BASE_URL = "https://www.openstreetmap.org/api/0.6/";
	public static final String RELATION_API_WAY_URL = "https://www.openstreetmap.org/api/0.6/way/";
	public static final String RELATION_API_NODE_URL = "https://www.openstreetmap.org/api/0.6/node/";

	public static final String RELATION_TYPE_VALUE_RELATION = "relation";
	public static final String RELATION_TYPE_VALUE_WAY      = "way";
	public static final String RELATION_TYPE_VALUE_NODE     = "node";

	public static final NSArray<String> parseableTypes = new NSArray<>(new String[] {
			RELATION_TYPE_VALUE_RELATION,
			RELATION_TYPE_VALUE_WAY
	});

	public static final NSArray<String> possibleTypes = new NSArray<>(new String[] {
			RELATION_TYPE_VALUE_RELATION,
			RELATION_TYPE_VALUE_WAY,
			RELATION_TYPE_VALUE_NODE
	});

	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(OsmRelation.class);

	public String url() {
		StringBuilder str = new StringBuilder();
		str.append(this.osmType());
		str.append("/");
		str.append(this.osmId());
		return str.toString();
	}

	public void setUrl(String nextUrl) {
		NSArray<String> parts = NSArray.componentsSeparatedByString(nextUrl, "/");
		String osmType = parts.get(0);
		Integer osmId = Integer.valueOf(parts.get(1));
		this.setOsmType(osmType);
		this.setOsmId(osmId);
	}

	public String fullUrl() {
		StringBuilder str = new StringBuilder();
		str.append("http://www.openstreetmap.org/");
		str.append(this.url());
		return str.toString();
	}

	public boolean isWay() {
		return "way".equals(this.osmType());
	}

	public boolean isRelation() {
		return "relation".equals(this.osmType());
	}

	public OsmRelationCheck lastCheck() {
		OsmRelationCheck lastCheck = null;
		for (OsmRelationCheck check : this.checks()) {
			if (lastCheck == null || 
					lastCheck.checkedTime() == null || 
					lastCheck.checkedTime().compareTo(check.checkedTime()) < 0) {
				lastCheck = check;
			}
		}
		return lastCheck;
	}

	public String lastUpdateDate() {
		OsmRelationCheck check = this.lastCheck();
		if (check != null) {
			String update = this.lastCheck().osmUpdateDate();
			if (update != null) {
				return update.substring(0, 10);
			} else {
				return null;
			}
		} else {
			return null;
		}
	}

	public Long nextPk(EOEditingContext ec) {
		@SuppressWarnings("unchecked")
		NSArray<NSDictionary<String,Long>> results = EOUtilities.rawRowsForSQL(ec, "Boundaries", "select max(pk) as maxPk from osm_relations", new NSArray<String>("maxPk"));
		return (Long)results.get(0).valueForKey("maxPk") + 1L;
	}

	public String condition() {
		StringBuilder str = new StringBuilder();
		for (OsmRelationCheck check : this.checks()) {
			str.append((check.checkResult() == OsmRelationCheck.CHECK_IS_GOOD) ? "+ " : "- ");
		}
		return str.toString();
	}
}
