package org.opencalaccess.courtcases.eo;

import java.util.Date;

import org.opencalaccess.U;
import org.opencalaccess.invites.eo.User;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.webobjects.eocontrol.EOEditingContext;
import com.webobjects.foundation.NSArray;
import com.webobjects.foundation.NSMutableArray;
import com.webobjects.foundation.NSTimestamp;

import er.extensions.eof.ERXQ;

public class CourtWorkloadEntry extends _CourtWorkloadEntry {

	private static final long serialVersionUID = -8604176845419341137L;

	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(CourtWorkloadEntry.class);

	public static final String allTestsPassedRecord = System.getProperty("CourtWorkloadEntryAllTestsPass");

	public static CourtWorkloadEntry createCourtWorkloadEntry(CourtPage courtPage, String userAddress) {

		EOEditingContext ec = courtPage.editingContext();

		User user = User.userForEmail(ec, userAddress);

		CourtWorkloadEntry entry = _CourtWorkloadEntry.createCourtWorkloadEntry(ec, U.now(), courtPage, user);

		return entry;
	}

	public static NSArray<CourtWorkloadEntry> entriesForUser(EOEditingContext ec, String email) {
		NSMutableArray<CourtWorkloadEntry> found = new NSMutableArray<>();
		for (CourtWorkloadEntry oneEntry : CourtWorkloadEntry.fetchAllCourtWorkloadEntries(ec)) {
			if (oneEntry.user().emailAddress().equals(email)) {
				found.add(oneEntry);
			}
		}
		return found.immutableClone();
//		return CourtWorkloadEntry.fetchCourtWorkloadEntries(ec, ERXQ.is(CourtWorkloadEntry.USER_KEY + "." + User.EMAIL_ADDRESS_KEY, email), null);
//		return CourtWorkloadEntry.fetchCourtWorkloadEntries(ec, CourtWorkloadEntry.USER.dot(User.EMAIL_ADDRESS).is(email), null);
	}

	public boolean allTestsPass() {
		return allTestsPassedRecord.equals(this.checksPassed());
	}

	public void setPageText(String pageText) {
		this.courtPage().setPageText(pageText);
	}

	public NSTimestamp createdDate() {
		return new NSTimestamp(new Date(this.created()));
	}

	public NSTimestamp completedDate() {
		if (this.completed() == null) {
			return null;
		} else {
			return new NSTimestamp(new Date(this.completed()));
		}
	}

	public boolean isPayable() {
		return (this.payment() == null || U.isNull(this.payment().paid())) && this.allTestsPass();
	}

	public void checkAll() {
		StringBuilder passed = new StringBuilder();
		passed.append(this.checkA());
		passed.append(this.checkB());
		passed.append(this.checkC());
		passed.append(this.checkD());
		passed.append(this.checkE());
		this.setChecksPassed(passed.toString());
	}

	/**
	 * Check the meta to make sure that this pageText is not empty and has not been used before.
	 */
	public String checkA() {

		String passed = "A"; String failed = "a";

		if (this.casesDigest() == null || this.casesDigest().isEmpty()) {
			return failed;
		}

		NSArray<CourtWorkloadEntry> found = CourtWorkloadEntry.fetchCourtWorkloadEntries(
				this.editingContext(),
				ERXQ.is(CourtWorkloadEntry.CASES_DIGEST_KEY, this.casesDigest())
					.and(ERXQ.not(ERXQ.is(this))),
				null);

		if (found.isEmpty()) {
			return passed;
		}

		for (CourtWorkloadEntry otherEntry : found) {
			if ( ! this.courtPage().calendarDateDept().department().equals(otherEntry.courtPage().calendarDateDept().department())) {
				return failed;
			}
			if ( ! this.courtPage().calendarDateDept().calendarDate().displayDate().equals(otherEntry.courtPage().calendarDateDept().calendarDate().displayDate())) {
				return failed;
			}
			if ( ! this.courtPage().calendarDateDept().calendarDate().calendar().name().equals(otherEntry.courtPage().calendarDateDept().calendarDate().calendar().name())) {
				return failed;
			}
		}

		return (found.isEmpty()) ? passed : failed;
	}

	public String checkB() {

		String passed = "B"; String failed = "b";

		String pageText = this.courtPage().pageText();

		if (pageText == null || pageText.isEmpty()) {
			return failed;
		}

		String calendarName = this.courtPage().calendarDateDept().calendarDate().calendar().calSpeak();

		return pageText.contains(calendarName) ? passed : failed;
	}

	public String checkC() {

		String passed = "C"; String failed = "c";

		if (this.casesDigest() == null || this.casesDigest().isEmpty()) {
			return failed;
		}

		NSArray<CourtWorkloadEntry> found = CourtWorkloadEntry.fetchCourtWorkloadEntries(
				this.editingContext(),
				ERXQ.is(CourtWorkloadEntry.CASES_DIGEST_KEY, this.pageDigest())
					.and(ERXQ.not(ERXQ.is(this))),
				null);

		return (found.isEmpty()) ? passed : failed;
	}

	public String checkD() {

		return null;

//		String passed = "D"; String failed = "d";
//
//		String pageText = this.courtPage().pageText();
//	
//		String foundDigest = this.pageDigest();
//
//		if (pageText == null || pageText.isEmpty() || foundDigest == null || foundDigest.isEmpty()) {
//			return failed;
//		}
//	
//		String derivedDigest = CourtCasesUtilities.md5Digest(this.courtPage().pageText());
//
//		return (foundDigest.equals(derivedDigest)) ? passed : failed;
	}

	public String checkE() {

		String passed = "E"; String failed = "e";

		return (this.courtPage().cases().size() > 0) ? passed : failed;
	}
}
