package org.opencalaccess.leginfo;

import java.io.ByteArrayInputStream;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

public class BillVersionDoc {

	private static final SAXParserFactory spf = SAXParserFactory.newInstance();

	static {
		spf.setNamespaceAware(true);
	}

	private static XMLReader xmlReader() {

	    SAXParser saxParser = null;
	    
	    try {
			saxParser = spf.newSAXParser();
		} catch (javax.xml.parsers.ParserConfigurationException e) {
			e.printStackTrace();
			saxParser = null;
		} catch (org.xml.sax.SAXException e) {
			e.printStackTrace();
			saxParser = null;
		}

	    XMLReader xmlReader = null;
	    try {
			xmlReader = saxParser.getXMLReader();
		} catch (org.xml.sax.SAXException e1) {
			e1.printStackTrace();
			xmlReader = null;
		}

	    return xmlReader;
	}

	public BillVersionDoc(String xml) {
		originalXml = xml;
	}

	private String originalXml;

	public String digestText() {

		XMLReader xmlReader = xmlReader();

	    DigestTextHandler handler = new DigestTextHandler();

	    xmlReader.setContentHandler(handler);

	    try {
			xmlReader.parse(new InputSource(new ByteArrayInputStream(originalXml.getBytes())));
		} catch (java.io.IOException e) {
			e.printStackTrace();
		} catch (org.xml.sax.SAXException e) {
			e.printStackTrace();
		}

	    return handler.digestText;
	}

	public static class DigestTextHandler extends DefaultHandler implements ContentHandler {

		StringBuilder str = null;

		public String digestText = null;

		@Override
		public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws org.xml.sax.SAXException {
			if (localName.equals("DigestText")) {
				str = new StringBuilder();
			}
			if (str != null && localName.equals("p")) {
				str.append("<p>");
			}
		}

		@Override
		public void endElement(String uri, String localName, String qName) throws org.xml.sax.SAXException {
			if (localName.equals("DigestText")) {
				digestText = str.toString();
				str = null;
			}
			if (str != null && localName.equals("p")) {
				str.append("</p>");
			}
		}

		@Override
		public void characters(char[] ch, int start, int length) throws org.xml.sax.SAXException {
			if (str != null) {
				String nextStr = new String(ch, start, length);
				str.append(nextStr);
			}
		}

		@Override
		public void processingInstruction(String target, String data) throws org.xml.sax.SAXException {

			if (str != null) {

				if (target.equals("xm-insertion_mark_start")) {
					str.append("<b>");
				}

				if (target.equals("xm-insertion_mark_end")) {
					str.append("</b>");
				}

				if (target.equals("xm-deletion_mark")) {
					String nextData = data;

					assert(nextData.startsWith("data=\""));

					nextData = nextData.substring("data=\"".length());

					assert(nextData.endsWith("\""));

					nextData = nextData.substring(0, nextData.length() - 1);

					nextData = nextData.replaceAll("\\x26", "<");
					nextData = nextData.replaceAll("<html:", "<");
					str.append("<s>");
					str.append(nextData);
					str.append("</s>");
				}
			}
		}
	}
}
