package org.opencalaccess.leginfo.eo;

import org.opencalaccess.U;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.webobjects.foundation.NSArray;
import com.webobjects.foundation.NSMutableArray;

import er.extensions.foundation.ERXArrayUtilities;

/**
 * An author is the author of a bill. The author could be a real person, or it could be
 * a committee.
 *
 * This is almost captured in the leginfo data but the committee authors in leginfo data
 * point to the committee members, but the committee members are not recognized as authors
 * themselves. It is helpful to have one layer of authors, as one finds here, and not a kind
 * of author that hides other authors behind it.
 *
 * @author ray
 */
public class Author extends _Author {

	private static final long serialVersionUID = -8155670541590669083L;

	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(Author.class);

	public String authorName() {
		return U.string(this.house().equals("A") ? "Asm " : "Sen ", this.name());
	}

	public NSArray<Bill> billsOfContribution(String contribution) {

		NSArray<Bill> found = new NSMutableArray<>();

		NSArray<Authoring> authorings = ERXArrayUtilities.filteredArrayWithQualifierEvaluation(
				this.authorings(),
				Authoring.CONTRIBUTION.is(contribution));

		for (Authoring authoring : authorings) {
			found.add(authoring.billVersionAuthor().billVersion().bill());
		}

		return found;
	}
}
