package org.opencalaccess.leginfo.eo;

import org.apache.log4j.Logger;

import com.webobjects.eocontrol.EOQualifier;
import com.webobjects.eocontrol.EOSortOrdering;
import com.webobjects.foundation.NSArray;

import er.extensions.eof.ERXQ;

public class Bill extends _Bill {

	private static final long serialVersionUID = 5666768719293066872L;

	private static Logger log = Logger.getLogger(Bill.class);

	/**
	 * The legislature is providing the measure_type and measure_num separately, but these are
	 * traditionally seen together as the "measure". The measure also includes information about
	 * whether this is from an extraordinary session in the prefix of the measure.
	 *
	 * Examples are "AB 1", "SB 22" and "ABx1 10", for the 10th Assembly Bill in Extraordinary
	 * Session 1.
	 *
	 * @return String form of the measure.
	 */
	public String measure() {
		if (this.measureType() == null || this.measureNum() == null) {
			log.error("Bad measure: type: \"" + this.measureType() + "\", num: \"" + this.measureNum() + "\"");
			return null;
		}
		if (this.sessionNum().equals("0")) {
			return this.measureType() + " " + this.measureNum();
		} else {
			return this.measureType() + "x" + this.sessionNum() + " " + this.measureNum();
		}
	}

	public NSArray<BillHistory> histories() {

		NSArray<BillHistory> foundHistories = super.histories();

		if (foundHistories == null || foundHistories.size() == 0) {
			return new NSArray<BillHistory>();
		}

		NSArray<String> transUpdates = foundHistories.valueForKey(BillHistory.TRANS_UPDATE_DT);

		String latestTransDate = transUpdates.get(0);
		
		for (String transDate : transUpdates) {
			if (transDate.compareTo(latestTransDate) > 0) {
				latestTransDate = transDate;
			}
		}

		NSArray<BillHistory> latestHistories = EOQualifier.filteredArrayWithQualifier(foundHistories, ERXQ.is(BillHistory.TRANS_UPDATE_DT_KEY, latestTransDate));

		return EOSortOrdering.sortedArrayUsingKeyOrderArray(latestHistories, BillHistory.actionSequenceKey.descs());
	}

	public BillHistory latestHistory() {
		return this.histories().get(0);
	}

	public DailyFileItem latestDailyFileItem() {

		NSArray<DailyFileItem> found = DailyFileItem.fetchDailyFileItems(
				this.editingContext(),
				ERXQ.and(
						DailyFileItem.BILL_ID.is(this.billId()),
						DailyFileItem.PUBLICATION_DATE.is(DailyFileItem.latestDailyFileDate())),
				null);

		return (found.size() > 0) ? found.get(0) : null;
	}
}
