package org.opencalaccess.leginfo.eo;

import org.apache.log4j.Logger;
import org.opencalaccess.leginfo.BillVersionDoc;

import com.webobjects.foundation.NSArray;
import com.webobjects.foundation.NSMutableArray;

public class BillVersion extends _BillVersion {

	private static final long serialVersionUID = -7706308441262071947L;

	@SuppressWarnings("unused")
	private static Logger log = Logger.getLogger(BillVersion.class);

	public boolean isLatestVersion() {
		return this.isLatest() == 1;
	}

	public NSArray<Authoring> leadAuthors() {

		NSArray<Authoring> found = new NSMutableArray<>();

		for (BillVersionAuthor versionAuthor : this.authors()) {
			for (Authoring authoring : versionAuthor.authorings()) {
				if (authoring.contribution().equals(Authoring.LEAD_AUTHOR)) {
					found.add(authoring);
				}
			}
		}

		return found;
	}

	public NSArray<Authoring> principalCoauthors() {

		NSArray<Authoring> found = new NSMutableArray<>();

		for (BillVersionAuthor versionAuthor : this.authors()) {
			for (Authoring authoring : versionAuthor.authorings()) {
				if (authoring.contribution().equals(Authoring.PRINCIPAL_COAUTHOR)) {
					found.add(authoring);
				}
			}
		}

		return found;
	}

	public NSArray<Authoring> coauthors() {

		NSArray<Authoring> found = new NSMutableArray<>();

		for (BillVersionAuthor versionAuthor : this.authors()) {
			for (Authoring authoring : versionAuthor.authorings()) {
				if (authoring.contribution().equals(Authoring.COAUTHOR)) {
					found.add(authoring);
				}
			}
		}

		return found;
	}

	public NSArray<Authoring> committeeChairAuthors() {
		NSArray<Authoring> found = new NSMutableArray<>();

		for (BillVersionAuthor versionAuthor : this.authors()) {
			for (Authoring authoring : versionAuthor.authorings()) {
				if (authoring.contribution().equals(Authoring.COMMITTEE_CHAIR)) {
					found.add(authoring);
				}
			}
		}

		return found;
	}

	public NSArray<Authoring> committeeViceChairAuthors() {
		NSArray<Authoring> found = new NSMutableArray<>();

		for (BillVersionAuthor versionAuthor : this.authors()) {
			for (Authoring authoring : versionAuthor.authorings()) {
				if (authoring.contribution().equals(Authoring.COMMITTEE_VICE_CHAIR)) {
					found.add(authoring);
				}
			}
		}

		return found;
	}

	public NSArray<Authoring> committeeMemberAuthors() {
		NSArray<Authoring> found = new NSMutableArray<>();

		for (BillVersionAuthor versionAuthor : this.authors()) {
			for (Authoring authoring : versionAuthor.authorings()) {
				if (authoring.contribution().equals(Authoring.COMMITTEE_MEMBER)) {
					found.add(authoring);
				}
			}
		}

		return found;
	}

	public String digestText() {
		
		String digest = (new BillVersionDoc(this.billXml())).digestText();

		if (digest == null) {
			return null;
		}

		digest = digest.replaceAll("\t", " ");

		digest = digest.replaceAll("\r\n", " ");

		digest = digest.replaceAll("\n", " ");

		digest = digest.replaceAll(" +", " ");

		return digest;
	}

	public NSArray<String> digestTextParts() {

		NSArray<String> found = new NSMutableArray<>();

		for (String part : NSArray.componentsSeparatedByString(this.digestText(), "<p>")) {

			if (part.endsWith("</p>")) {
				part = part.substring(0, part.length() - "</p>".length());
			}

			boolean foundBs = false;

			int startB = -1;
			int endB = -1;

			do {
				foundBs = false;

				startB = part.indexOf("<b>", startB);
				endB = part.indexOf("</b>", startB);

				if (startB > 0 && endB < startB) {
					part = part + "</b>";
				}

//				if (startB < 0 && endB > 0) {
//					nextText = "<b>" + nextText;
//					foundBs = true;
//					startB = endB;
//				}

				if (startB > 0 && endB > startB) {
					foundBs = true;
					startB = endB;
				}

			} while (foundBs);

			boolean foundSs = false;

			int startS = -1;
			int endS = -1;

			do {
				foundSs = false;

				startS = part.indexOf("<s>", startS);
				endS = part.indexOf("</s>", startS);

				if (startS > 0 && endS < startS) {
					part = part + "</s>";
				}

//				if (startS < 0 && endS > 0) {
//					nextText = "<s>" + nextText;
//					foundSs = true;
//					startS = endS;
//				}

				if (startS > 0 && endS > startS) {
					foundSs = true;
					startS = endS;
				}

			} while (foundSs);

			if ( ! part.isEmpty()) {
				found.add(part);
			}
		}
		return found.immutableClone();
	}

	/**
	 * Return the &lt;caml:DigestText&gt; section of a biil, with each string being
	 * a paragraph, and insertion marks and removals turned into bold and strikethrough
	 * tags.
	 *
	 * @return NSArray<String> sentences
	 */
	public NSArray<String> thisBillWoulds() {

		NSArray<String> nextTexts = new NSMutableArray<>();

		for (String textPara : this.digestTextParts()) {

			if ( ! textPara.isEmpty()) {

				// TODO come up with some better way of finding "this bill would" phrases.
				//
				boolean firstMatch = textPara.matches(".*[Tt]his.*bill.*would.*");
				boolean secondMatch = textPara.matches(".*[Tt]he.*bill.*would.*");

				if (firstMatch || secondMatch) {
					nextTexts.add(textPara);
				}
			}
		}

		return nextTexts.immutableClone();
	}
}
