package org.opencalaccess.leginfo.eo;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.webobjects.eoaccess.EOUtilities;
import com.webobjects.eocontrol.EOSortOrdering;
import com.webobjects.foundation.NSArray;
import com.webobjects.foundation.NSDictionary;

import er.extensions.eof.ERXEC;

public class DailyFileItem extends _DailyFileItem {

	private static final long serialVersionUID = -2444202886289291164L;

	private static String latestDailyFileDate;

	public static String latestDailyFileDate() {

		if (latestDailyFileDate == null) {

			@SuppressWarnings("unchecked")
			NSArray<NSDictionary<String, Object>> results = EOUtilities.rawRowsForSQL(
					ERXEC.newEditingContext(),
					"LegData",
					"select max(publication_date) from daily_file_tbl",
					new NSArray<String>("publicationDate"));

			latestDailyFileDate = results.get(0).valueForKey("publicationDate").toString().substring(0, 10);
		}
		return latestDailyFileDate;
	}

	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(DailyFileItem.class);

	public NSArray<LocationCode> locationCodes() {
		NSArray<LocationCode> found = LocationCode.fetchLocationCodes(
				this.editingContext(),
				LocationCode.LOCATION_CODE.is(this.locationCode()),
				null);
		return found;
	}

	public boolean isConsent() {
		return this.consentCalendarCode() != null || this.locationCodeObj().description().contains("Consent");
	}

	public LocationCode locationCodeObj() {

		NSArray<LocationCode> found = this.locationCodes();

		if (found == null || found.size() == 0) {
			return null;
		}

		if (found.size() > 1) {
			return EOSortOrdering.sortedArrayUsingKeyOrderArray(found, LocationCode.TRANS_UPDATE.descs()).get(0);
		}

		return found.get(0);
	}
}
